/******************************************************************************/
/* User Level #define Macros                                                  */
/******************************************************************************/

//comment out the TV that is not being used. This will affect which tv can be
//controlled through RF commands
//#define SAMSUNG
#define SAMSUNG


#define TRUE 1
#define FALSE 0

#define MAX_BUFFSIZE 1
#define MAX_COMMAND_LENGTH 400

#define BUFF_FULL 1
#define BUFF_EMPTY 0xFF
#define BUFF_NORMAL 0

#ifdef SAMSUNG
#define NUM_IR_COMMANDS 14
#endif
#ifdef PANASONIC
#define NUM_IR_COMMANDS 5
#endif

#define NUM_RF_COMMANDS 14

//pulse widths for S(hort) and L(ong): take number and multiply by granularity (0.016ms = .000016s)
#define RF_S_PULSE_WIDTH_MIN 31
#define RF_S_PULSE_WIDTH_MAX 42
#define RF_L_PULSE_WIDTH_MIN 107
#define RF_L_PULSE_WIDTH_MAX 118



/******************************************************************************/
/* User Function Prototypes                                                   */
/******************************************************************************/

//flags
unsigned char COMMAND_OVERFLOW = 0;
unsigned int INDEX = 0;   //index into the current command
unsigned char REC = 0;    //recording in progress
unsigned char REC_PASS = 0; //the number of IR 'packets' that have been recorded
unsigned char WAITING_FOR_REC = 0; //1 if waiting for first pulse from IR reciever
unsigned char PLAY = 0;   //playback in progress
unsigned char PLAY_PASS = 0; //the number of times the sequence has been played back
unsigned char COMMAND_DONE = 1;  //command has completed recording
unsigned char PLAY_DONE = 0;     //command is done being played back
unsigned char RF_FIRST_EDGE;
unsigned char RF_COMMAND_DONE;

char IR_STAT;   //1 if the LED is on, 0 if its off

//Buffer used for transmitting and recieving data
typedef struct _BUFFER {
  int head;
  int tail;
  unsigned char data[MAX_BUFFSIZE];
} BUFFER;


// int COMMAND1[MAX_COMMAND_LENGTH];
unsigned int TEMP_COMMAND[MAX_COMMAND_LENGTH]; //IR COMMAND
unsigned int TEMP_COMMAND_SIZE = 0;   //the size of the currently stored command

unsigned char RF_COMMAND[16]; //the temp RF command
unsigned char RF_COMMAND_POS; //current temp rf command array index

/* TODO User level functions prototypes (i.e. InitApp) go here */

void InitApp(void);         /* I/O and Peripheral Initialization */
int BUFF_push(BUFFER*, char);
char BUFF_pop(BUFFER*);
int BUFF_status(BUFFER*);

void clear_temp_command();
char load_command(unsigned int);
void clear_rf_command();
char get_rf_command();
int READ_FLASH(int *);


//---------------------STORED RF COMMANDS FROM BASE STATION------------------
const char RFCMD_TV_POWER[16] =     {'S', 'S','L','L','S','S','L','L','S','S','S','S','S','L','L','L'};
const char RFCMD_TV_VOL_UP[16] =    {'S', 'S','L','L','S','S','L','L','S','S','S','S','L','S','L','L'};
const char RFCMD_TV_VOL_DOWN[16] =  {'S', 'S','L','L','S','S','L','L','S','S','S','S','L','L','L','L'};
const char RFCMD_TV_CHAN_UP[16] =   {'S', 'S','L','L','S','S','L','L','S','S','S','L','S','S','L','L'};
const char RFCMD_TV_CHAN_DOWN[16] = {'S', 'S','L','L','S','S','L','L','S','S','S','L','S','L','L','L'};
const char RFCMD_TV_UP[16] =        {'S', 'S','L','L','S','S','L','L','S','S','S','L','L','S','L','L'};
const char RFCMD_TV_DOWN[16] =      {'S', 'S','L','L','S','S','L','L','S','S','S','L','L','L','L','L'};
const char RFCMD_TV_LEFT[16] =      {'S', 'S','L','L','S','S','L','L','S','S','L','S','S','S','L','L'};
const char RFCMD_TV_RIGHT[16] =     {'S', 'S','L','L','S','S','L','L','S','S','L','S','S','L','L','L'};
const char RFCMD_TV_ENTER[16] =     {'S', 'S','L','L','S','S','L','L','S','S','L','S','L','S','L','L'};
const char RFCMD_TV_RETURN[16] =    {'S', 'S','L','L','S','S','L','L','S','S','L','S','L','L','L','L'};
const char RFCMD_TV_SOURCE[16] =    {'S', 'S','L','L','S','S','L','L','S','S','L','L','S','S','L','L'};
const char RFCMD_TV_MENU[16] =      {'S', 'S','L','L','S','S','L','L','S','S','L','L','S','L','L','L'};
const char RFCMD_TV_GUIDE[16] =     {'S', 'S','L','L','S','S','L','L','S','S','L','L','L','S','L','L'};

//RF commands for controlling the TVs
const char* RF_COMMANDS[NUM_RF_COMMANDS] = {RFCMD_TV_POWER, RFCMD_TV_VOL_UP,RFCMD_TV_VOL_DOWN,RFCMD_TV_CHAN_UP,RFCMD_TV_CHAN_DOWN,RFCMD_TV_UP,RFCMD_TV_DOWN,RFCMD_TV_LEFT,RFCMD_TV_RIGHT,RFCMD_TV_ENTER,RFCMD_TV_RETURN,RFCMD_TV_SOURCE,RFCMD_TV_MENU,RFCMD_TV_GUIDE};






//---------------------STORED IR COMMANDS---------------------------
const unsigned int POWER_PANASONIC[101] = {10,14077, 6782, 1957, 1537, 1956, 5033, 1958, 1536, 1957, 1537, 1956, 1538, 1956, 1538,
1956, 1538, 1955, 1539, 1955, 1539, 1956, 1538, 1954, 1540, 1955, 1539, 1957, 1537, 1956, 5032, 1957, 1537,
1957, 1537, 1957, 1537, 1956, 1537, 1956, 1538, 1957, 1537, 1956, 1538, 1955, 1539, 1955, 1539, 1956, 5032,
1956, 1538, 1957, 1537, 1956, 1538, 1956, 1538, 1956, 1538, 1956, 1538, 1956, 1538, 1956, 1538, 1956, 5032,
1956, 1538, 1957, 5032, 1959, 5030, 1959, 5030, 1957, 5031, 1959, 1535, 1958, 1536, 1957, 5032, 1957, 1537,
1957, 5032, 1958, 5030, 1958, 5030, 1959, 5030, 1959, 1535, 1957, 5031, 1958,0};

const unsigned int VOL_UP_PANASONIC[101] = {10, 13916, 6942, 1978, 1516, 1986, 4999, 2006,
1489, 2013, 1481, 1981, 1513, 1989, 1505, 1998, 1496, 2007, 1487, 1974, 1519, 1983, 1511, 1992, 1502, 2000, 1494, 2008, 1485,
1977, 5009, 1996, 1499, 2004, 1490, 2012, 1482, 1979, 1514, 1988, 1506, 1997, 1497, 2005, 1488, 1973, 1520, 1982, 1511, 1991,
4994, 2011, 1485, 1977, 1518, 1985, 1509, 1993, 1501, 2001, 1493, 2009, 1484, 1977, 1516, 1986, 1507, 1995, 1499, 2003, 1490,
1972, 1522, 1980, 1513, 1989, 1505, 1998, 4988, 1977, 1518, 1984, 1510, 1992, 1502, 2000, 1493, 2009, 1485, 1977, 1517, 1985,
1509, 1994, 4991, 2014, 1481, 1980, 5006, 2000, 0};

const unsigned int VOL_DOWN_PANASONIC[101] = {10,14047, 6814, 1925, 1569, 1925, 5064, 1925, 1569, 1925, 1569, 1925, 1569, 1925,
1569, 1923, 1571, 1920, 1574, 1923, 1572, 1923, 1571, 1923, 1571, 1923, 1572, 1922, 1572, 1922, 5066, 1925,
1569, 1924, 1570, 1924, 1571, 1922, 1572, 1922, 1572, 1922, 1572, 1922, 1572, 1921, 1573, 1922, 1572, 1923,
5065, 1924, 1570, 1924, 1571, 1924, 1570, 1924, 1570, 1924, 1571, 1923, 1571, 1923, 1571, 1923, 1571, 1923,
5065, 1925, 1569, 1925, 1569, 1922, 1572, 1923, 1571, 1923, 5066, 1924, 1570, 1923, 1571, 1921, 5067, 1926,
1569, 1925, 1570, 1924, 1571, 1923, 1571, 1923, 5066, 1926, 1568, 1925, 5063, 1926,0};

const unsigned int CHAN_UP_PANASONIC[101] = {10,14051, 6810, 1931, 1563, 1930, 5058, 1932, 1562, 1930, 1564, 1931, 1563,
1929, 1565, 1931, 1563, 1931, 1563, 1931, 1563, 1930, 1564, 1930, 1564, 1930, 1564, 1931, 1563, 1931, 5057,
1935, 1560, 1933, 1561, 1932, 1562, 1933, 1561, 1933, 1561, 1932, 1562, 1932, 1562, 1931, 1563, 1932, 1562,
1932, 5056, 1935, 1560, 1934, 1560, 1933, 1561, 1932, 1562, 1932, 1562, 1932, 1562, 1932, 1562, 1932, 1562,
1932, 1562, 1932, 1562, 1933, 5056, 1933, 1561, 1933, 5056, 1935, 5054, 1935, 1560, 1932, 1562, 1933, 1561,
1932, 1562, 1932, 5056, 1934, 1561, 1930, 5059, 1933, 5056, 1933, 1561, 1933, 5056, 1931,0};

const unsigned int CHAN_DOWN_PANASONIC[101] = {10,14059, 6802, 1937, 1557, 1936, 5053, 1938, 1556, 1937, 1557, 1937, 1557, 1936,
1558, 1936, 1558, 1935, 1559, 1935, 1559, 1935, 1559, 1935, 1559, 1936, 1558, 1933, 1561, 1933, 5055, 1935,
1559, 1936, 1559, 1935, 1559, 1932, 1562, 1934, 1560, 1934, 1560, 1934, 1560, 1934, 1560, 1933, 1561, 1932,
5056, 1934, 1560, 1933, 1561, 1931, 1563, 1933, 1561, 1933, 1561, 1931, 1563, 1934, 1560, 1934, 1560, 1932,
5056, 1933, 1562, 1933, 5055, 1935, 1560, 1934, 5055, 1932, 5057, 1934, 1560, 1933, 1561, 1933, 5056, 1935,
1559, 1934, 5055, 1934, 1560, 1934, 5055, 1935, 5053, 1934, 1560, 1934, 5055, 1932,0};


//SAMSUNG_TV
const unsigned int POWER_SAMSUNG[69] = {10, 17998, 17837, 2487, 6419, 2460, 6446, 2473, 6433, 2486, 1968, 2267, 2186, 2253, 2200, 2279, 2173, 2265, 2187, 2251, 6652, 2471, 6436, 2484, 6422, 2497, 1957, 2278, 2175, 2264, 2189, 2250, 2203, 2276, 2176, 2263, 2190, 2249, 6654, 2469, 1986, 2250, 2204, 2276, 2177, 2262, 2191, 2248, 2205, 2275, 2178, 2261, 6642, 2481, 1973, 2262, 6643, 2480, 6426, 2494, 6413, 2466, 6441, 2479, 6428, 2492, 6415, 2464,0};

const unsigned int VOL_UP_SAMSUNG[69] = {10, 17968, 17860, 2464, 6440, 2479, 6426, 2494, 6411, 2467, 1986, 2494, 1959, 2480, 1972, 2467, 1985, 2495, 1957, 2278, 6626, 2497, 6407, 2471, 6434, 2485, 1969, 2470, 1983, 2252, 2201, 2279, 2173, 2266, 2186, 2253, 6650, 2472, 6432, 2487, 6417, 2461, 1993, 2283, 2170, 2269, 2183, 2256, 2197, 2283, 2169, 2269, 2184, 2255, 2197, 2283, 2169, 2269, 6634, 2285, 6619, 2463, 6442, 2477, 6429, 2490, 6415, 2464,0};

const unsigned int VOL_DOWN_SAMSUNG[69] = {10, 17991, 17839, 2486, 6419, 2500, 6405, 2474, 6431, 2489, 1965, 2474, 1979, 2460, 1992, 2488, 1964, 2474, 1978, 2462, 6443, 2477, 6427, 2492, 6413, 2466, 1988, 2492, 1961, 2478, 1974, 2465, 1987, 2289, 2164, 2275, 6630, 2493, 6412, 2467, 1987, 2493, 6410, 2468, 1985, 2250, 2202, 2277, 2175, 2264, 2189, 2250, 2202, 2277, 2175, 2264, 6641, 2278, 2174, 2265, 6640, 2280, 6625, 2457, 6448, 2472, 6433, 2486,0};

const unsigned int CHAN_UP_SAMSUNG[69] = {10, 17979, 17855, 2468, 6438, 2481, 6425, 2494, 6411, 2467, 1987, 2492, 1961, 2478, 1975, 2464, 1989, 2491, 1961, 2274, 6630, 2493, 6411, 2467, 6437, 2482, 1972, 2466, 1987, 2249, 2204, 2276, 2176, 2262, 2190, 2249, 2204, 2276, 6629, 2494, 1959, 2276, 2177, 2262, 6642, 2480, 1974, 2261, 2192, 2288, 2165, 2274, 6630, 2493, 1960, 2275, 6629, 2494, 6412, 2466, 1989, 2287, 6617, 2261, 6645, 2478, 6429, 2491,0};

const unsigned int CHAN_DOWN_SAMSUNG[69] = {10, 17996, 17840, 2484, 6423, 2496, 6411, 2468, 6439, 2480, 1974, 2465, 1989, 2490, 1963, 2273, 2180, 2259, 2194, 2286, 6617, 2465, 6440, 2479, 6426, 2493, 1961, 2478, 1975, 2260, 2193, 2287, 2166, 2273, 2179, 2260, 2192, 2288, 2165, 2274, 2178, 2262, 2191, 2248, 6655, 2468, 1986, 2250, 2203, 2277, 2176, 2263, 6640, 2483, 6422, 2498, 6408, 2471, 6435, 2485, 1970, 2265, 6639, 2484, 6421, 2498, 6408, 2470,0};

const unsigned int UP_SAMSUNG[69] = {10, 17976, 17858, 2466, 6441, 2479, 6428, 2491, 6416, 2463, 1992, 2488, 1966, 2473, 1980, 2459, 1994, 2282, 2170, 2269, 6636, 2487, 6419, 2460, 6447, 2473, 1982, 2457, 1997, 2279, 2174, 2265, 2188, 2251, 2202, 2277, 2176, 2263, 2190, 2249, 2204, 2275, 2178, 2262, 2191, 2289, 6617, 2466, 6441, 2479, 1976, 2259, 6647, 2476, 6431, 2488, 6419, 2460, 6448, 2471, 6436, 2483, 1971, 2264, 2190, 2249, 6657, 2263,0};

const unsigned int DOWN_SAMSUNG[69] = {10, 17991, 17845, 2479, 6427, 2493, 6414, 2465, 6442, 2478, 1977, 2463, 1991, 2489, 1964, 2475, 1979, 2461, 1992, 2488, 6419, 2461, 6445, 2475, 6431, 2488, 1966, 2473, 1980, 2459, 1994, 2282, 2172, 2267, 2186, 2254, 6654, 2266, 2187, 2252, 2201, 2279, 2175, 2265, 2189, 2250, 6658, 2262, 6645, 2478, 1977, 2258, 2196, 2284, 6625, 2254, 6653, 2267, 6640, 2484, 6424, 2496, 1958, 2277, 2176, 2263, 6643, 2276,0};

const unsigned int LEFT_SAMSUNG[69] = {10, 18002, 17838, 2487, 6420, 2499, 6409, 2470, 6437, 2482, 1972, 2466, 1987, 2493, 1960, 2479, 1974, 2465, 1989, 2491, 6417, 2461, 6446, 2474, 6434, 2486, 1969, 2470, 1984, 2496, 1957, 2278, 2176, 2263, 2190, 2249, 6659, 2260, 2193, 2286, 6621, 2462, 1992, 2284, 2169, 2270, 6637, 2282, 6625, 2458, 1997, 2279, 2174, 2265, 6642, 2277, 2177, 2263, 6644, 2275, 6632, 2288, 2167, 2272, 2181, 2258, 6649, 2271,0};

const unsigned int RIGHT_SAMSUNG[69] = {10, 17989, 17851, 2474, 6434, 2486, 6421, 2497, 6409, 2470, 1985, 2495, 1959, 2480, 1974, 2465, 1988, 2287, 2166, 2273, 6633, 2490, 6416, 2462, 6444, 2475, 1981, 2458, 1996, 2280, 2173, 2266, 2187, 2252, 2202, 2278, 2176, 2263, 6644, 2276, 2179, 2261, 2193, 2287, 2166, 2273, 6634, 2286, 6621, 2462, 1993, 2283, 6622, 2256, 2198, 2281, 6624, 2255, 6652, 2267, 6640, 2279, 2176, 2263, 2191, 2289, 6617, 2261,0};

const unsigned int ENTER_SAMSUNG[69] = {10, 17986, 17859, 2465, 6444, 2476, 6433, 2486, 6423, 2497, 1959, 2480, 1974, 2465, 1989, 2491, 1963, 2272, 2182, 2257, 6651, 2472, 6434, 2485, 6422, 2497, 1958, 2480, 1974, 2261, 2193, 2287, 2166, 2272, 2181, 2258, 2196, 2284, 2170, 2269, 2185, 2254, 6654, 2266, 2188, 2251, 6656, 2263, 6644, 2479, 1977, 2259, 6649, 2271, 6637, 2282, 6626, 2252, 2203, 2277, 6630, 2289, 2166, 2273, 2180, 2259, 6648, 2272,0};

const unsigned int RETURN_SAMSUNG[69] = {10,17986, 17859, 2465, 6443, 2477, 6431, 2488, 6421, 2498, 1957, 2482, 1973, 2467, 1987, 2492, 1961, 2274, 2179, 2260, 6648, 2476, 6431, 2488, 6420, 2500, 1955, 2484, 1970, 2265, 2188, 2251, 2203, 2277, 2177, 2262, 2192, 2288, 2166, 2273, 2180, 2259, 6649, 2474, 6434, 2486, 1970, 2265, 6641, 2482, 1974, 2262, 6644, 2479, 6429, 2490, 6418, 2460, 1995, 2281, 2174, 2265, 6641, 2279, 2176, 2263, 6643, 2276,0};

const unsigned int SOURCE_SAMSUNG[69] = {10, 17988, 17855, 2469, 6439, 2479, 6429, 2490, 6418, 2500, 1954, 2485, 1969, 2470, 1983, 2497, 1957, 2482, 1972, 2508, 6400, 2478, 6429, 2490, 6418, 2256, 2198, 2485, 1970, 2468, 1983, 2496, 1958, 2482, 1972, 2466, 6441, 2274, 2180, 2503, 1952, 2487, 1966, 2473, 1981, 2499, 1955, 2483, 1970, 2469, 1985, 2495, 1959, 2480, 6428, 2287, 6621, 2257, 6651, 2268, 6641, 2278, 6631, 2288, 6620, 2258, 6650, 2269,0};

const unsigned int MENU_SAMSUNG[69] = {10, 18012, 17833, 2287, 6620, 2258, 6651, 2269, 6640, 2279, 2177, 2262, 2192, 2288, 2167, 2272, 2182, 2257, 2198, 2282, 6624, 2254, 6654, 2266, 6643, 2277, 2178, 2261, 2193, 2287, 2167, 2272, 2182, 2257, 2197, 2283, 2170, 2268, 6638, 2282, 2173, 2266, 6641, 2279, 6630, 2289, 2167, 2272, 2183, 2256, 2198, 2281, 6625, 2254, 2202, 2277, 6630, 2290, 2165, 2274, 2180, 2259, 6649, 2270, 6639, 2281, 6628, 2292,0};

const unsigned int GUIDE_SAMSUNG[69] = {10, 17978, 17861, 2465, 6442, 2478, 6428, 2492, 6415, 2464, 1990, 2491, 1962, 2477, 1976, 2464, 1990, 2491, 1963, 2477, 6430, 2491, 6416, 2464, 6442, 2478, 1975, 2464, 1989, 2492, 1961, 2478, 1975, 2465, 1989, 2492, 6414, 2465, 6441, 2480, 6426, 2494, 1960, 2480, 6427, 2493, 6414, 2466, 1988, 2492, 6415, 2465, 1989, 2491, 1962, 2273, 2180, 2260, 6647, 2477, 1977, 2259, 2194, 2286, 6621, 2463, 1990, 2490,0};





//IR COMMANDS AND ENUMERATION - only one TV can be controlled at a time. Comment out the array for the TV that won't be used
#ifdef SAMSUNG
//smasung ir commands and enum
typedef enum IR_ENUM_t_ {
    POWER_SAMSUNG_CMD,
    VOL_UP_SAMSUNG_CMD,
    VOL_DOWN_SAMSUNG_CMD,
    CHAN_UP_SAMSUNG_CMD,
    CHAN_DOWN_SAMSUNG_CMD,
    UP_SAMSUNG_CMD,
    DOWN_SAMSUNG_CMD,
    LEFT_SAMSUNG_CMD,
    RIGHT_SAMSUNG_CMD,
    ENTER_SAMSUNG_CMD,
    RETURN_SAMSUNG_CMD,
    SOURCE_SAMSUNG_CMD,
    MENU_SAMSUNG_CMD,
    GUIDE_SAMSUNG_CMD
} IR_ENUM_t;
IR_ENUM_t CURRENT_COMMAND;
//--samsung tv commands
const unsigned int* IR_COMMANDS[14] = {POWER_SAMSUNG,VOL_UP_SAMSUNG,VOL_DOWN_SAMSUNG,CHAN_UP_SAMSUNG,CHAN_DOWN_SAMSUNG,UP_SAMSUNG,DOWN_SAMSUNG,LEFT_SAMSUNG,RIGHT_SAMSUNG,ENTER_SAMSUNG,RETURN_SAMSUNG,SOURCE_SAMSUNG,MENU_SAMSUNG,GUIDE_SAMSUNG};
#endif

#ifdef PANASONIC
//panasonic commands and enum
typedef enum IR_ENUM_t_ {
    POWER_PANASONIC_CMD,
    VOL_UP_PANASONIC_CMD,
    VOL_DOWN_PANASONIC_CMD,
    CHAN_UP_PANASONIC_CMD,
    CHAN_DOWN_PANASONIC_CMD
} IR_ENUM_t;
IR_ENUM_t CURRENT_COMMAND;
//--panasonic tv commands
const unsigned int* IR_COMMANDS[5] = {POWER_PANASONIC,VOL_UP_PANASONIC,VOL_DOWN_PANASONIC,CHAN_UP_PANASONIC,CHAN_DOWN_PANASONIC};
#endif


